import React, { useRef, useEffect, useState } from 'react';
import { Map, NavigationControl, Marker, Source, Layer } from 'react-map-gl/maplibre';
import { MapPin, Star, Navigation, Loader, Sun, Moon } from 'lucide-react';
import { useTranslation } from 'react-i18next';
import { LocationData, StarConditions } from '../types';

interface MapViewProps {
  location: LocationData;
  onLocationChange: (location: LocationData) => void;
  conditions: StarConditions | null;
}

const MapView: React.FC<MapViewProps> = ({ location, onLocationChange, conditions }) => {
  const { t } = useTranslation();
  const mapRef = useRef<any>();
  const [viewState, setViewState] = useState({
    longitude: location.lng,
    latitude: location.lat,
    zoom: 10
  });
  const [isLocating, setIsLocating] = useState(false);

  useEffect(() => {
    setViewState(prev => ({
      ...prev,
      longitude: location.lng,
      latitude: location.lat
    }));
  }, [location]);

  // Get current location with timezone detection
  const getCurrentLocation = () => {
    setIsLocating(true);
    if (navigator.geolocation) {
      navigator.geolocation.getCurrentPosition(
        (position) => {
          const { latitude, longitude } = position.coords;
          
          // Get timezone-aware address
          const timezone = Intl.DateTimeFormat().resolvedOptions().timeZone;
          const localTime = new Date().toLocaleString(undefined, { timeZone: timezone });
          
          const newLocation: LocationData = {
            lat: latitude,
            lng: longitude,
            address: `${latitude.toFixed(4)}, ${longitude.toFixed(4)} (${timezone})`
          };
          onLocationChange(newLocation);
          setIsLocating(false);
        },
        (error) => {
          console.error('Error getting location:', error);
          setIsLocating(false);
          alert(t('map.locating') + ' ' + error.message);
        },
        { enableHighAccuracy: true, timeout: 10000, maximumAge: 60000 }
      );
    } else {
      alert('Geolocation not supported');
      setIsLocating(false);
    }
  };

  const handleMapClick = async (event: any) => {
    const { lng, lat } = event.lngLat;
    
    const newLocation: LocationData = {
      lat,
      lng,
      address: `${lat.toFixed(4)}, ${lng.toFixed(4)}`
    };
    
    onLocationChange(newLocation);
  };

  const getScoreColor = (score: number) => {
    if (score >= 4) return '#22c55e'; // Green
    if (score >= 3) return '#eab308'; // Yellow
    if (score >= 2) return '#f97316'; // Orange
    return '#ef4444'; // Red
  };

  // Enhanced day/night overlay with timezone awareness
  const createDayNightOverlay = () => {
    const now = new Date();
    const utcHours = now.getUTCHours();
    const utcMinutes = now.getUTCMinutes();
    const sunLng = ((utcHours + utcMinutes / 60) * 15) - 180;
    
    // Create multiple polygons for better night visualization
    const nightPolygons = [];
    
    // Night side (opposite to sun)
    const nightCenterLng = sunLng + 180;
    
    // Create a more accurate night representation
    for (let i = 0; i < 4; i++) {
      const offsetLng = nightCenterLng + (i - 2) * 90;
      nightPolygons.push({
        type: 'Feature' as const,
        geometry: {
          type: 'Polygon' as const,
          coordinates: [[
            [offsetLng - 45, -90],
            [offsetLng + 45, -90],
            [offsetLng + 45, 90],
            [offsetLng - 45, 90],
            [offsetLng - 45, -90]
          ]]
        },
        properties: { type: 'night' }
      });
    }
    
    return {
      type: 'FeatureCollection' as const,
      features: nightPolygons
    };
  };

  // Enhanced terminator line (day/night boundary)
  const createTerminatorLine = () => {
    const now = new Date();
    const utcHours = now.getUTCHours();
    const utcMinutes = now.getUTCMinutes();
    const sunLng = ((utcHours + utcMinutes / 60) * 15) - 180;
    
    const terminatorPoints = [];
    for (let lat = -90; lat <= 90; lat += 5) {
      terminatorPoints.push([sunLng + 90, lat]);
    }
    
    const terminatorPoints2 = [];
    for (let lat = -90; lat <= 90; lat += 5) {
      terminatorPoints2.push([sunLng - 90, lat]);
    }
    
    return {
      type: 'FeatureCollection' as const,
      features: [
        {
          type: 'Feature' as const,
          geometry: {
            type: 'LineString' as const,
            coordinates: terminatorPoints
          },
          properties: { type: 'terminator' }
        },
        {
          type: 'Feature' as const,
          geometry: {
            type: 'LineString' as const,
            coordinates: terminatorPoints2
          },
          properties: { type: 'terminator' }
        }
      ]
    };
  };

  const dayNightLayer: any = {
    id: 'day-night',
    type: 'fill',
    source: 'day-night',
    paint: {
      'fill-color': '#0f0f23',
      'fill-opacity': 0.7
    }
  };

  const terminatorLayer: any = {
    id: 'terminator',
    type: 'line',
    source: 'terminator',
    paint: {
      'line-color': '#fbbf24',
      'line-width': 2,
      'line-opacity': 0.8
    }
  };

  // Determine if current location is in night or day using timezone
  const isNightTime = () => {
    const now = new Date();
    const utcHours = now.getUTCHours();
    const utcMinutes = now.getUTCMinutes();
    const sunLng = ((utcHours + utcMinutes / 60) * 15) - 180;
    const nightCenterLng = sunLng + 180;
    
    // Normalize longitude difference
    let diff = Math.abs(location.lng - nightCenterLng);
    if (diff > 180) diff = 360 - diff;
    
    return diff < 90;
  };

  const currentTimeType = isNightTime();

  // Get current local time for the location
  const getCurrentLocalTime = () => {
    try {
      // Estimate timezone from longitude (rough approximation)
      const estimatedTimezone = Math.round(location.lng / 15);
      const utcTime = new Date();
      const localTime = new Date(utcTime.getTime() + (estimatedTimezone * 60 * 60 * 1000));
      return localTime.toLocaleTimeString(undefined, { hour: '2-digit', minute: '2-digit' });
    } catch {
      return new Date().toLocaleTimeString(undefined, { hour: '2-digit', minute: '2-digit' });
    }
  };

  return (
    <div className="relative h-96 rounded-xl overflow-hidden border border-slate-700/50">
      <Map
        ref={mapRef}
        {...viewState}
        onMove={evt => setViewState(evt.viewState)}
        onClick={handleMapClick}
        style={{ width: '100%', height: '100%' }}
        mapStyle={{
          version: 8,
          sources: {
            'satellite': {
              type: 'raster',
              tiles: [
                'https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}'
              ],
              tileSize: 256,
              attribution: '© Esri'
            }
          },
          layers: [
            {
              id: 'satellite',
              type: 'raster',
              source: 'satellite'
            }
          ]
        }}
        attributionControl={false}
      >
        <NavigationControl position="top-right" />
        
        {/* Enhanced Day/Night overlay */}
        <Source id="day-night" type="geojson" data={createDayNightOverlay()}>
          <Layer {...dayNightLayer} />
        </Source>
        
        {/* Terminator line */}
        <Source id="terminator" type="geojson" data={createTerminatorLine()}>
          <Layer {...terminatorLayer} />
        </Source>
        
        {/* Current location marker */}
        <Marker longitude={location.lng} latitude={location.lat}>
          <div className="relative">
            <div 
              className="w-8 h-8 rounded-full border-2 border-white shadow-xl flex items-center justify-center animate-pulse-slow"
              style={{ backgroundColor: conditions ? getScoreColor(conditions.starScore) : '#6b7280' }}
            >
              <Star className="h-4 w-4 text-white" fill="currentColor" />
            </div>
            {conditions && (
              <div className="absolute -top-10 left-1/2 transform -translate-x-1/2 bg-slate-900/90 text-white px-3 py-1 rounded-lg text-sm font-medium whitespace-nowrap border border-slate-600">
                ★ {conditions.starScore.toFixed(1)}
              </div>
            )}
          </div>
        </Marker>
      </Map>

      {/* Enhanced current location button */}
      <button
        onClick={getCurrentLocation}
        disabled={isLocating}
        className="absolute top-4 left-4 bg-gradient-to-r from-purple-600 to-blue-600 hover:from-purple-700 hover:to-blue-700 disabled:from-gray-600 disabled:to-gray-700 backdrop-blur-sm rounded-xl px-4 py-3 text-white font-medium transition-all duration-200 shadow-lg border border-white/20 flex items-center space-x-2"
      >
        {isLocating ? (
          <Loader className="h-4 w-4 animate-spin" />
        ) : (
          <Navigation className="h-4 w-4" />
        )}
        <span className="text-sm">
          {isLocating ? t('map.locating') : t('map.currentLocation')}
        </span>
      </button>

      {/* Enhanced location info overlay */}
      <div className="absolute bottom-4 left-4 right-4 bg-slate-900/80 backdrop-blur-sm rounded-xl p-4 text-white border border-slate-700/50">
        <div className="flex items-center space-x-2 mb-2">
          <MapPin className="h-4 w-4 text-purple-400" />
          <span className="text-sm font-medium">{location.address}</span>
        </div>
        
        {/* Current time status */}
        <div className="flex items-center justify-between text-xs mb-3">
          <div className="flex items-center space-x-2">
            {currentTimeType ? (
              <>
                <Moon className="h-4 w-4 text-blue-300" />
                <span className="text-blue-300 font-medium">{t('map.currentlyNight')}</span>
              </>
            ) : (
              <>
                <Sun className="h-4 w-4 text-yellow-400" />
                <span className="text-yellow-400 font-medium">{t('map.currentlyDay')}</span>
              </>
            )}
          </div>
          <span className="text-gray-400">
            {getCurrentLocalTime()}
          </span>
        </div>
        
        {/* Day/Night legend */}
        <div className="flex items-center justify-between text-xs">
          <div className="flex items-center space-x-1">
            <div className="w-3 h-3 bg-slate-800 rounded-full border border-slate-600"></div>
            <span className="text-blue-300">{t('map.nightRegion')}</span>
          </div>
          <div className="flex items-center space-x-1">
            <div className="w-3 h-3 bg-yellow-400 rounded-full"></div>
            <span className="text-yellow-300">{t('map.dayRegion')}</span>
          </div>
          <div className="flex items-center space-x-1">
            <div className="w-6 h-0.5 bg-yellow-400 rounded-full"></div>
            <span className="text-yellow-300">{t('map.dayNightBoundary')}</span>
          </div>
        </div>
      </div>
    </div>
  );
};

export default MapView;